
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * HEADER ir.h
 *
 * PURPOSE:
 *	This header file is the general IR include file.  
 *
 * SCOPE:
 *	All IR CSCs must include this header file to ensure that the
 *	correct system wide types are being used.
 *	
 * NOTES:
 *	Character pointers specified in any structure element are null 
 *	terminated ASCII strings unless a count is associated with it.
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 * 
 ****************************************************************************/

#ifndef IR_H

#define IR_H

/* IR Basic types */
typedef char 		CHAR;
typedef unsigned char 	UCHAR;
typedef short 		SHORT;
typedef unsigned short	USHORT;
typedef long 		LONG;
typedef unsigned long	ULONG;
#ifndef BOOLEAN
typedef SHORT	BOOLEAN;
#endif

#ifdef XVT_MSDOS
#undef XVT_MSDOS
#endif

#ifdef BSDSOCKET
#undef BSDSOCKET
#endif

#ifdef WINSOCKET
#undef WINSOCKET
#endif

#ifdef WINPATHWAY
#undef WINPATHWAY
#endif

#ifdef AMICUS
#undef AMICUS
#endif

#ifdef MS_WINDOWS
#undef MS_WINDOWS
#endif

/* ASN.1 run-time library configuration */
#ifdef USE_MIN_BUF
#undef USE_MIN_BUF
#endif
#ifdef USE_EXP_BUF
#undef USE_EXP_BUF
#endif

#ifdef USE_SBUF
#undef USE_SBUF
#endif 

/* Host system libraries (especially used for memcheck utilities) */
#ifdef MEM_STDLIB
#undef MEM_STDLIB
#endif

#ifdef MEM_XVTLIB
#undef MEM_XVTLIB
#endif

/* Disable Memory checking (used for memcheck utilities) */
#ifdef IR_MEM_CHECK
#undef IR_MEM_CHECK
#endif

/* System configuration */
#define WINSOCKET
#define USE_SBUF
#define XVT_MSDOS
#define MEM_XVTLIB

#ifndef MS_WINDOWS
#define HANDLE		USHORT
#endif

/* Maximum malloc size for any IR block of memory */
#define IR_MAX_MALLOC_SIZE		0xFFFF

/* 
 * Memory malloc method used for APDU encode/decode and 
 * snacc run-time library  
 */
#ifdef IR_USE_IR_MALLOC
#undef IR_USE_IR_MALLOC
#endif

#ifdef IR_USE_ASN_ALLOC
#undef IR_USE_ASN_ALLOC
#endif

#define IR_USE_IR_MALLOC


#define IR_LEN_1000		1000
#define IR_LEN_500		500
#define IR_LEN_100		100
#define IR_LEN_50		50
#define IR_LEN_20		20

/****************************************************************************
 * System Types:
 ****************************************************************************/


typedef int		FD;		/*  file descriptors  */

/* 
 * A variable length building block for most ASN.1 types used by 
 * this application. A null pointer will indicate that the optional
 * data string is not present.
 */
typedef struct 
{
	USHORT	len;		/* Length of string. */
	CHAR	*sPtr;		/* pointer to string */
	BOOLEAN	isMalloced;	/* is IR_ASN_BASE_STRUCT malloced */
} IR_ASN_BASE_STRUCT;

/*
 * Octet String.
 */
typedef IR_ASN_BASE_STRUCT	OCTET_STRING;

/*
 * Object Identifier. An object identifier is must be specified
 * in a dotted string format (ie. 1.2.840.10003.7.1).
 */
typedef IR_ASN_BASE_STRUCT	OBJECT_IDENTIFIER;

/*
 * Visible string is an array of characters. Null terminated.
 */
typedef IR_ASN_BASE_STRUCT	VISIBLE_STRING;

/*
 * Bit string.
 */
typedef IR_ASN_BASE_STRUCT	BIT_STRING;

/*
 * Basic ASN list structure used to build a variable length list
 * of elements.
 */
typedef struct IR_LIST_ELEM
{
	struct IR_LIST_ELEM	*next;	/* Next element in list */
	struct IR_LIST_ELEM	*prev;	/* Previous element in list */
	void			(*destFn)();
	void			*(*dupFn)(); /* dup function returns pointer to dupped elem data. */
	void			*elem;	/* pointer to current elem data */
	BOOLEAN			isMalloced;
} IR_LIST_ELEM;


typedef struct IR_LIST
{
	IR_LIST_ELEM		*first;	/* First element in list */
	IR_LIST_ELEM		*last;	/* Last element in list */
	IR_LIST_ELEM		*current; /* Current element in list */
	SHORT			size;	/* Number of elements in list */
	BOOLEAN			isMalloced;	/* Is this structure malloced? */
} IR_LIST;


/* External reference types */
typedef SHORT	IR_EXT_REF_TYPE;
#define IR_EXT_REF_NO_REF	0
#define IR_EXT_REF_DIRECT	1
#define IR_EXT_REF_INDIRECT	2
#define IR_EXT_REF_DVD		3

/* External encoding types */
typedef SHORT	IR_EXT_ENC_TYPE;
#define IR_EXT_ENC_NONE		0
#define IR_EXT_ENC_ANY		1
#define IR_EXT_ENC_ALIGNED	2
#define IR_EXT_ENC_ARBITRARY	3

typedef struct
{
	IR_EXT_REF_TYPE		referenceType;
	OBJECT_IDENTIFIER	direct;
	LONG			indirect;
	VISIBLE_STRING		dataValueDesc;
	IR_EXT_ENC_TYPE		encType;
	USHORT			anyType;	/* Used if encType == ANY */
	void			*any;
	OCTET_STRING		octetAligned;
	BIT_STRING		arbitrary;
	BOOLEAN			isMalloced;
} IR_EXTERNAL;


/****************************************************************************
 * IR Common Constants and Types:
 ****************************************************************************/

#define IR_CONFIG_FILE "ir.cfg"
#define IR_MAX_LINE_SIZE 80


/*
 * End of string marker.
 */
#define EOS			'\0'

/*
 * IR Status.
 */
typedef SHORT	IR_STATUS;

#define IR_SUCCESS		0
#define IR_FAIL			1

/*
 * IR Error levels and codes.
 */
typedef SHORT	IR_ERROR_LEVEL;		/* no error, warning, memory, fatal */
typedef SHORT	IR_ERROR_CODE;		/* Some unique IR number */

#define IR_ERROR_NO_ERROR	0
#define IR_ERROR_WARNING	1
#define IR_ERROR_FATAL		3

/*
 *  Boolean type (TRUE or FALSE).
 */

#define TRUE		1
#define FALSE		0

/*
 * Communication Event reply error code for error status only.
 */
typedef SHORT 	IR_COMM_EVENT_ERROR_CODE;

/* Error codes are provided by the communication service starting at base. */
#define IR_COMM_EVENT_ERROR_BASE	100;

/*
 * Communication Event reply status type.
 */
typedef SHORT 	IR_COMM_EVENT_STATUS;

/*
 * Common Cummunication event status for API, ALI and PLI.
 */
#define IR_COMM_DATA_ERROR			(-1)	/* error */
#define IR_COMM_DATA_NO_DATA			0	/* no data */
#define IR_COMM_DATA_CONNECT     	    	1	/* connect ack */
#define IR_COMM_DATA_RELEASE			2	/* release ack */
#define IR_COMM_DATA_CONN_REFUSED		14	/* Connect refused by target */
#define IR_COMM_DATA_UNKNOWN_HOST		20	/* Couldn't resolve name/addr */
/*
 * PLI interface only.
 */
#define IR_COMM_DATA_APDU			4	/* apdu data */
#define IR_COMM_DATA_A_ABORT			5	/* abort apdu. not used */
#define IR_COMM_DATA_A_P_ABORT			6	/* abort detected */
/*
 * ALI and API interfaces only.
 */
#define IR_COMM_DATA_ABORT			3	/* abort ack */
#define IR_COMM_DATA_INIT_SRVC			7	/* init service */
#define IR_COMM_DATA_SEARCH_SRVC	    	8	/* search service */
#define IR_COMM_DATA_PRESENT_SRVC		9	/* present service */
#define IR_COMM_DATA_DELETE_SRVC	    	10	/* delete service */
#define IR_COMM_DATA_ACCS_SRVC			11	/* access service */
#define IR_COMM_DATA_RSRC_CNTL_SRVC	    	12	/* resource service */
#define IR_COMM_DATA_RSRC_RPT_SRVC	    	13	/* resource report */
/* Kernel detected protocol error and is aborting assoc */
#define IR_COMM_DATA_PROTOCOL_ERR		15	

/* 
 * API interface only. For informative use only. No need to act on these  
 * other than to generate some useful message to the user.
 */
#define IR_COMM_DATA_ABORT_FAIL			16	/* couldn't abort */
#define IR_COMM_DATA_INIT_FAIL			17	/* Couldn't process init */
#define IR_COMM_DATA_CONN_FAIL			18	/* Couldn't send init req */
#define IR_COMM_DATA_REL_FAIL			19	/* Couldn't release */
#define IR_COMM_DATA_SRCH_FAIL			21	/* Couldn't process search response */
#define IR_COMM_DATA_PRSNT_FAIL			22	/* Couldn't process present */
#define IR_COMM_DATA_HOST_NOT_FOUND		23	/* Address or name resolution failure */

/*
 * Communication Configuation.
 */
typedef SHORT	IR_COMM_CONFIGURATION;

#define IR_COMM_CONF_EVENT_INT		0
#define IR_COMM_CONF_EVENT_POLLED	1

/*
 * Language Configuration.
 */
typedef SHORT	IR_LANG_CONFIGURATION;

#define IR_ENGLISH			0
#define IR_FRENCH			1
	
/*
 * Invalid or unknown identifiers.
 */
#define IR_UNKNOWN_ID_TYPE		(-1)

/*
 * User Identifier.
 */
typedef SHORT 	IR_USER_ID;

/*
 * Association Identifier.
 */
typedef SHORT	IR_ASSOC_ID;

/*
 * Work space identifier.
 */
typedef SHORT 	IR_WS_ID;

/*
 * Target identifier.
 */
typedef SHORT	IR_TARGET_ID;

/*
 * WorkSpace Search Identifier.
 */
typedef SHORT	IR_SEARCH_ID;

/*
 * WorkSpace Query Identifier.
 */
typedef SHORT	IR_QUERY_ID;

/*
 * WorkSpace record Identifier.
 */
typedef SHORT	IR_RECORD_ITEM_ID;

/*
 * Event structure.
 */
typedef struct
{
	IR_ASSOC_ID	assocId;
	USHORT		event;
	USHORT		wParam;
	ULONG		lParam;
	BOOLEAN		isMalloced;
} IR_COMM_EVENT;

/*******************************************************************
 * User and Target Information Tables. The information supplied in
 * these tables is based on information specified in the Explain
 * Facility for SR.
 *******************************************************************/

/* User privileges. */
typedef SHORT	IR_USER_PRIV_TYPE;

#define IR_USER_PRIV_ADMIN	0	/* Administrator */
#define IR_USER_PRIV_REG	1	/* Regular user */

/* Database attribute Set Id. */
typedef SHORT	IR_ATTR_SET_ID;

/* Target options. */
typedef SHORT	IR_TARGET_OP_TYPE;

/* Record syntax */
typedef SHORT	IR_DB_REC_SYNTAX;

#define IR_TARGET_OP_SEARCH		0x0001
#define IR_TARGET_OP_PRESENT		0x0002
#define IR_TARGET_OP_DELETE_SET		0x0004
#define IR_TARGET_OP_RESOURCE_REPORT	0x0008
#define IR_TARGET_OP_TRIG_RSRC_CNTL	0x0010
#define IR_TARGET_OP_RESOURCE_CNTL	0x0020
#define IR_TARGET_OP_ACCESS_CONTROL	0x0040

/* 
 * User Access Information.
 */
typedef struct 
{
	VISIBLE_STRING		name;
	VISIBLE_STRING		password;
	IR_USER_PRIV_TYPE	privilege;
	BOOLEAN			isMalloced;
} IR_USER_ACCESS_INFO;

/*
 * The interface language preference and accounting information
 * is seperate for each user.
 */
typedef struct
{
	VISIBLE_STRING		name;			/* User name */
	IR_LANG_CONFIGURATION	ifLangPref;		/* Interface language
							 * preference. 
							 */
	OCTET_STRING		accountCreditInfo;	/* Future implementation. */
	BOOLEAN			isMalloced;
} IR_USER_GEN_INFO;

/*
 * A target structure is required for each known target for
 * each user as well as a general default set.
 */
typedef struct
{
	IR_TARGET_ID		targetId;
	VISIBLE_STRING		targetName;
	VISIBLE_STRING		userAuthentication;
	LONG			defSmallSetUpperBound;
	LONG			defLargeSetLowerBound;
	LONG			defPresentNumber;
	OBJECT_IDENTIFIER	defPreferedRecSyntax;
	VISIBLE_STRING		defElementSetName;
	BOOLEAN			defResultSetNamingSupported;
	VISIBLE_STRING		displayFormats;
	BOOLEAN			isMalloced;
} IR_USER_TARGET_INFO;


/****
 * Target information.
 ***/

/*
 * This information will be provided for each known target.
 */
typedef struct
{
	IR_TARGET_ID	targetId;
	VISIBLE_STRING	applCntxName;
	VISIBLE_STRING	applProcessTitle;
	VISIBLE_STRING	applEntityQual;
	VISIBLE_STRING	ipAddress;
	VISIBLE_STRING	ipDomainName;
	USHORT		portNum;
	BOOLEAN		isMalloced;
} IR_TARGET_ASSOC_INFO;

	
typedef struct
{
	IR_TARGET_ID		targetId;
	VISIBLE_STRING		targetName;
	VISIBLE_STRING		description;
	BIT_STRING		options;
	VISIBLE_STRING		contactInfo;
	BOOLEAN			resultSetNamingSupported;
	BOOLEAN			elementSetNamingSupported;
	SHORT			maxNumResultSets;
	SHORT			hoursOfOperation;
	VISIBLE_STRING		chargingPolicy;
	IR_LIST			dbInfo;		/* list of IR_DATABASE_INFO */
	VISIBLE_STRING		idAuthentication;
	BOOLEAN			isMalloced;
} IR_TARGET_INFO;


/******
 * Database Information. This structure identifies one database only.
 * A target can have one or more databases.
******/
typedef struct
{
	IR_TARGET_ID		targetId;
	VISIBLE_STRING		databaseName;
	VISIBLE_STRING		description;
	VISIBLE_STRING		chargingPolicy;
	IR_LIST			attrSetId; /* List of OIDs */
	IR_LIST			recordSyntaxList; /* List of OIDs types */
	IR_LIST			elementSetNames;	/* Visible String */
	BOOLEAN			isMalloced;
} IR_DATABASE_INFO;

/*************************************************************************
 * Query and present types.
 *************************************************************************/

/*
 * Query types defined in ANSI Z39.50.
 */
typedef SHORT	IR_QUERY_TYPE;

#define IR_QUERY_TYPE_0			0
#define IR_QUERY_TYPE_1			1	/* RPN Query */
#define IR_QUERY_TYPE_2			2
#define IR_QUERY_TYPE_100		100
#define IR_QUERY_TYPE_101		101

/*
 * Present Status.
 */
typedef LONG	IR_PRESENT_STATUS;

#define IR_PRSNT_SUCCESS		0
#define IR_PRSNT_PARTIAL_1		1
#define IR_PRSNT_PARTIAL_2		2
#define IR_PRSNT_PARTIAL_3		3
#define IR_PRSNT_PARTIAL_4		4
#define IR_PRSNT_DELETE_FAILURE		5
#define IR_PRSNT_NO_STATUS		(-1)
/*
 * Result Set Status.
 */
typedef LONG IR_RESULT_SET_STATUS;

#define IR_RESULT_SET_NO_STATUS		(-1)
#define IR_RESULT_SET_SUBSET		1
#define IR_RESULT_SET_INTERIM		2
#define IR_RESULT_SET_NONE		3

/*
 * Query Operator types.
 */
typedef SHORT	IR_QUERY_OPERATOR;

#define IR_QUERY_OPER_AND			0
#define IR_QUERY_OPER_OR			1
#define IR_QUERY_OPER_AND_NOT			2

/*
 * Query Element types. 
 */
typedef SHORT IR_QUERY_ELEM_TYPE;

#define IR_QE_ATTR_SET_PLUS_TERM	0
#define IR_QE_RESULT_SET_ID		1
#define IR_QE_OPERATOR			2

/*************************************************************************
 * Query Structures.
 *************************************************************************/

/*
 * Element Set Names. A list of element set names or 
 * element set names and database names.
 */
#define IR_ELEM_SET_NAME_NONE		0
#define IR_ELEM_SET_NAME_GENERIC	1
#define IR_ELEM_SET_NAME_SPECIFIC	2

typedef SHORT IR_ELEM_SET_NAME_TYPE;

typedef struct 
{
	VISIBLE_STRING		dbName;		/* database string */
	VISIBLE_STRING		elemSetName;	/* set name */
	BOOLEAN			isMalloced;
} IR_SPECIFIC_ELEM_SET_NAME;

typedef struct
{
	VISIBLE_STRING		generic;
	IR_LIST			dbSpecific;	/* List of IR_SPECIFIC_ELEM_SET_NAME */
} IR_ELEM_SET_NAMES;

/*
 * Attributes
 */
typedef struct
{
	LONG		attrType;
	LONG		attrValue;
	BOOLEAN		isMalloced;
} IR_ATTR_ELEM;

/*
 * Attributes plus term.
 */
typedef struct
{
	IR_LIST		attrList;	/* List of attribute elements */
	OCTET_STRING	term;
	BOOLEAN		isMalloced;
} IR_ATTR_PLUS_TERM;
/*
 * Simple node element.
 */
#define IR_RPN_SIMPLE_ATTR		1
#define IR_RPN_SIMPLE_RESULT_SET_ID	2

typedef struct
{
	SHORT			simpleType;	/* Attr Plus term / result set id */
	IR_ATTR_PLUS_TERM	attrPlusTerm;
	VISIBLE_STRING		resultSetId;
	BOOLEAN			isMalloced;
} IR_RPN_SIMPLE_NODE;
	

/*
 *  Complex node element.
 */
typedef struct
{
	struct IR_RPN_ELEMENT	*rpnElem1;
	struct IR_RPN_ELEMENT	*rpnElem2;
	IR_QUERY_OPERATOR	operator;
	BOOLEAN			isMalloced;
} IR_RPN_COMPLEX_NODE;

#define IR_RPN_NODE_SIMPLE		1
#define IR_RPN_NODE_COMPLEX		2

typedef struct
{
	SHORT			rpnElemType;	/* Complex or simple. */
	IR_RPN_SIMPLE_NODE	*rpnOperand;
	IR_RPN_COMPLEX_NODE	*rpnStructSeq;
	BOOLEAN			isMalloced;
} IR_RPN_ELEMENT;
/*
 * RPN Query type.
 */
typedef struct	
{
	OBJECT_IDENTIFIER	attributeSetId;
	IR_RPN_ELEMENT		rpnElem;
	BOOLEAN			isMalloced;
} IR_RPN_QUERY_STRUCT;


/*************************************************************************
 * Present Record structures.
 *************************************************************************/

/*
 * Record types.
 */
typedef SHORT IR_RECORD_TYPE;

#define IR_REC_TYPE_NONE		(-1)
#define IR_REC_TYPE_SURR_DIAG		0
#define IR_REC_TYPE_DB_RECORD		1
#define IR_REC_TYPE_DB_NON_SURR		2


/*
 * Diagnostic record structure.
 */
typedef struct
{
	OBJECT_IDENTIFIER	diagnosticSetId;
	LONG			condition;	/* as per the specified 
						 * diagnostic set 
						 */
	VISIBLE_STRING		addInfo;	/* as per the specified
						 * diagnostic set 
						 */
	BOOLEAN			isMalloced;
} IR_DIAG_RECORD;

/*
 * Database record.
 */
typedef struct
{
	OBJECT_IDENTIFIER		dbRecSyntax;
	OCTET_STRING			record;
	BOOLEAN				isMalloced;
} IR_DB_RECORD;

/*
 * Name Plus record structure.
 */
typedef struct 
{
	IR_RECORD_TYPE			recordType;
	VISIBLE_STRING			databaseName;
	IR_DB_RECORD			dbRecord;
	IR_DIAG_RECORD			diagRecord;
	BOOLEAN				isMalloced;
} IR_RECORD_STRUCT;
	
/****************************************************************************
 * Delete Service types:
 ****************************************************************************/

/*
 * Delete set function.
 */
typedef SHORT	IR_DELETE_SET_FUNCTION;

#define IR_DELETE_SPECIFIC_SET		0
#define IR_DELETE_ALL_SETS		1

/*
 * Delete set status.
 */
typedef LONG	IR_DELETE_SET_STATUS;

#define IR_DEL_SUCCESS			0
#define IR_DEL_DID_NOT_EXIST		1
#define IR_DEL_ALREADY_DELETED		2
#define IR_DEL_TARGET_SYS_ERROR		3
#define IR_DEL_ACCESS_NOT_ALLOWED	4
#define IR_DEL_RESOURCE_CNTL		6
#define IR_DEL_BULK_DEL_NOT_SUPTD	7
#define IR_DEL_ALL_RESULTS_DELETED	8
#define IR_DEL_REQ_RESULTS_DELETED	9

/*
 * Delete status list.
 */
typedef struct 
{
	VISIBLE_STRING		resultSetId;
	IR_RESULT_SET_STATUS	resultSetStatus;
	IR_WS_ID		wsId;
	BOOLEAN			isMalloced;
} IR_DELETE_STATUS_STRUCT;

/****************************************************************************
 * IR Access and Resource Control Service types.
 ****************************************************************************/

/*
 * Trigger Control operation types.
 */
typedef SHORT	IR_TRIGGER_OPER_TYPE;

#define IR_TRIG_TYPE_RESOURCE_REPORT	1
#define IR_TRIG_TYPE_RESOURCE_CNTL	2
#define IR_TRIG_TYPE_CANCEL		3

/*
 * Resource Report Status.
 */
typedef SHORT IR_RESOURCE_REPORT_STATUS;

#define IR_RSRC_REP_SUCCESS		0
#define IR_RSRC_REP_PARTIAL		1
#define IR_RSRC_REP_FAILURE_1		2
#define IR_RSRC_REP_FAILURE_2		3
#define IR_RSRC_REP_FAILURE_3		4
#define IR_RSRC_REP_FAILURE_4		5
	
#endif
