
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 ************************************************************************/

/****************************************************************************
 * HEADER ok.h
 *
 * PURPOSE:
 *	This header file is the Origin Kernel header file. 
 *
 * SCOPE:
 *	All Origin Kernel functions must include this file.
 *
 * NOTES:
 *
 * $Header$
 * $Log$
 * 
 ****************************************************************************/

#ifndef OK_H

#define OK_H 1 
#include <stddef.h>
#include <ir.h>
#include <apdu.h>

#define MAX_EVENTS		22	
#define	MAX_STATES		10
#define MAX_RESPONSES		10
#define POPST			1000

#define OK_INVALID		0
#define OK_CLOSED   		1
#define	OK_INIT			2
#define OK_OPEN	   		3		
#define	OK_SEARCH		4
#define OK_PRESENT		5
#define OK_DELETE		6
#define	OK_RES_REPORT		7
#define OK_RES_CONTROL		8
#define OK_ACCESS_CONTROL	9
#define OK_RELEASE		10

#define OK_INIT_REQ 		1
#define OK_INIT_ACC_RES		2
#define OK_INIT_REJ_RES		3
#define OK_SEARCH_REQ		4
#define OK_SEARCH_RES		5
#define OK_PRESENT_REQ		6
#define OK_PRESENT_RES		7
#define OK_DELETE_REQ		8
#define OK_DELETE_RES		9
#define OK_RSRC_REP_REQ		10
#define OK_RSRC_REP_RES		11
#define OK_TRIG_CNTRL_REQ	12
#define OK_RSRC_R_CNTRL_REQ	13
#define OK_RSRC_NR_CNTRL_REQ	14
#define OK_RSRC_CNTRL_RES	15
#define OK_ACCESS_CNTRL_REQ	16
#define OK_ACCESS_CNTRL_RES	17
#define OK_A_ABORT_RES		18
#define OK_A_P_ABORT_RES	19
#define OK_I_ABORT_REQ		20
#define OK_RELEASE_REQ		21	
#define OK_RELEASE_RES		22

#define OK_CONNECT		25
#define OK_NO_DATA		30
#define OK_APDU			40

/****************************************************************************
 * State Table Information
 ****************************************************************************/

typedef struct ASSOC_DATA_TYPE
{
	IR_ASSOC_ID	assocId;
	SHORT		validEvents [MAX_EVENTS];
	SHORT		currentState;
	SHORT		previousState;
	struct ASSOC_DATA_TYPE	*nextAssocPtr; 
} ASSOC_DATA_TYPE;
 
typedef struct WAIT_FOR_CON_TYPE
{
	IR_ASSOC_ID	assocId;
	struct WAIT_FOR_CON_TYPE *nextAssocPtr;
} WAIT_FOR_CON_TYPE;

/****************************************************************************
 * State Table Functions
 ****************************************************************************/

IR_STATUS OkAse_CheckEvent( IR_ASSOC_ID assocId,
			   SHORT event );

IR_STATUS OkAse_UpdateState( IR_ASSOC_ID assocId,
			    SHORT event );

IR_STATUS OkAse_CreateAssoc( IR_ASSOC_ID assocId);

IR_STATUS OkAse_CheckAssoc( IR_ASSOC_ID assocId);

IR_STATUS OkAse_DeleteAssoc( IR_ASSOC_ID assocId);

IR_STATUS OkAse_DeleteAllAssocs();

IR_STATUS OkAse_InitAssoc ( IR_ASSOC_ID assocId, BIT_STRING *services);

IR_STATUS OkAse_ReceiveResponse(IR_ASSOC_ID assocId, APDU *apdu);

/****************************************************************************
 * Association Functions
 ****************************************************************************/

IR_STATUS OkAcse_Association (IR_ASSOC_ID assocId);

IR_STATUS OkAcse_AbortAssoc (IR_ASSOC_ID assocId);

IR_STATUS OkAcse_AAbort (IR_ASSOC_ID assocId);

IR_STATUS OkAcse_ReleaseAssoc (IR_ASSOC_ID assocId);

IR_STATUS OkAcse_ARelease (IR_ASSOC_ID assocId);

IR_STATUS OkAcse_Terminate ();

/****************************************************************************
 * Control Functions
 ****************************************************************************/

IR_STATUS OkCf_UpdateErrorReport (
	IR_ERROR_LEVEL level, IR_ERROR_CODE code, CHAR *source);
	
IR_STATUS OkAcse_StopConnectWait(IR_ASSOC_ID assocId);
BOOLEAN	  OkCf_IsIRKInitialized();
#endif

