
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 ************************************************************************/

/****************************************************************************
 * HEADER ir_op.h
 *
 * PURPOSE:
 *	This header file is global to the Origin Process CSC only.
 *
 * SCOPE:
 *	
 * NOTES:
 *
 * 
 ****************************************************************************/

#ifndef IR_OP_H

#define IR_OP_H
#include <ir.h>
#include <apdu.h>
#include <ir_api.h>
#include <rescio.h>

/*
 * Association state types.
 */
#define OP_ASSOC_CONNECTING		1
#define OP_ASSOC_INITIALIZING		2
#define OP_ASSOC_IDLE			3
#define OP_ASSOC_SEARCHING		4
#define OP_ASSOC_PRESENTING		5
#define OP_ASSOC_DELETING		6
#define OP_ASSOC_RSRC_REPORT		7
#define OP_ASSOC_RELEASING		8
#define OP_ASSOC_ABORTING		9
#define OP_ASSOC_UNASSOCIATED		10

typedef SHORT OP_ASSOC_STATE;

#define	OP_ID_SIZE	10

/*
 * Association/Target features that can be tested for using a
 * boolean yes/no TRUE/FALSE query on association management.
 * OpAm_AssocAllowed();
 */
#define OPAM_SEARCH		1
#define OPAM_PRESENT		2
#define OPAM_DELETE		3
#define OPAM_ACCESS		4
#define OPAM_RSRC_CTRL		5
#define OPAM_RSRC_REPORT	6
#define OPAM_TRIGGER		7
#define OPAM_RESULT_NAMING	8
#define OPAM_ELEM_SET_NAMING	9

/*
 * Association information table.
 */
typedef struct 
{
	IR_USER_ID		userId;
	IR_ASSOC_ID		assocId;	/* must always be present */
	IR_TARGET_ID		targetId;	/* Invalid if unknown */
	SHORT			securityParameterType; /* for access response */
	BOOLEAN			rsrcCntlRespReq;/* true if response required */
	VISIBLE_STRING		hostName;	/* required if address is not */
	VISIBLE_STRING		address;	/* required if hostName is not*/
	VISIBLE_STRING		localName;	/* required if targetId is not*/
	OP_ASSOC_STATE		state;		/* always something */
	IRO_COMM_INIT_RESP	*initResp;	/* target supplied info */
	VISIBLE_STRING		refId;		/* access/resource control use */
	VISIBLE_STRING		wsRefId;	/* WS Id of current req */
	BOOLEAN			resultNaming;	/* Option allowed by target? */
	BOOLEAN			elementSetNaming; /* Option allowed by target? */
	IR_LIST			*resultList;	/* Used for SR Delete msgs */
	IR_DELETE_SET_FUNCTION	delFunc;	/* USed for SR delete msgs */
	BOOLEAN			isMalloced;
} OP_ASSOC_ENTRY;

typedef struct 
{
	BIT_STRING	protocolVersion; 	
	BIT_STRING	options;
	LONG		preferredMessageSize;
	LONG		maximumRecordSize;	/* Optional */
	VISIBLE_STRING	implementationId;	/* Optional */
	VISIBLE_STRING	implementationName;	/* Optional */
	VISIBLE_STRING	implementationVersion;  /* Optional */
	BOOLEAN		isMalloced;
} OP_INIT_REQ_INFO;


/*
 * WorkSpace information
 */
typedef struct OP_WS_INFO
{
	IR_WS_ID			wsId;
	VISIBLE_STRING			resultSetName;
	IR_USER_ID			userId;
	IR_ASSOC_ID			assocId;
	LONG				totalTargetRecords;
	LONG				numWSRecords;
	LONG				nextTargetRec;
	LONG				numRecReturned;
	BOOLEAN				searchStatus;
	IR_RESULT_SET_STATUS		resultSetStatus;
	IR_PRESENT_STATUS		presentStatus;
	IR_DIAG_RECORD			nonSurrDiag;
	BOOLEAN				targetResultsDeleted;
	BOOLEAN				presenting;
	BOOLEAN				deleting;
	IR_LIST				records;	/* list of IR_RECORD_STRUCT */
	BOOLEAN				isMalloced;
} OP_WS_INFO;


#define OP_USER_NAME	50
#define OP_PASSWORD	51
#define OP_PRIVILEGE	52
#define OP_PREF_LANG	53
#define OP_CREDIT	54
#define OP_TRGT_FILE	55

#define OP_TRGT_ID	60
#define OP_TRGT_NAME 	61
#define OP_USER_AUTH	62
#define OP_UPR_BND	63
#define OP_LWR_BND	64
#define OP_PRSNT_NUM	65
#define OP_REC_SYN	66
#define OP_SET_NAME	67
#define OP_DSPLY_FMT	68
#define OP_DESC		69
#define	OP_OPTIONS	70
#define OP_CNTCT_INFO	71
#define OP_RSLT_SET	72
#define	OP_MAX_RSLT_SET	73
#define OP_HRS_OP	74
#define OP_CHRG_POLICY	75
#define OP_ID_AUTH	77
#define OP_CNTX_NAME	78
#define OP_PRCSS_TITLE	79
#define OP_ENTTY_QUAL	80
#define OP_IP_ADDR	81
#define OP_IP_DMN_NAME	82
#define OP_PORT_NUM	83

#define OP_DB_NAME	90
#define OP_ATTR_SET_ID	91

typedef struct 
{
	IR_USER_ID		userId;
	VISIBLE_STRING		userName;
	VISIBLE_STRING		password;
	IR_USER_PRIV_TYPE	privilege;
	IR_LANG_CONFIGURATION	prefLang;
	OCTET_STRING		accountCreditInfo;
	VISIBLE_STRING		targetInfoFile;
	BOOLEAN			useWorkSpace;
	BOOLEAN			isMalloced;
} OP_SESSION_INFO;

typedef struct
{
	IR_TARGET_ID		targetId;
	VISIBLE_STRING		targetName;
	VISIBLE_STRING		userAuthentication;
	LONG			defSmallSetUpperBound;
	LONG			defLargeSetLowerBound;
	LONG			defPresentNumber;
	OBJECT_IDENTIFIER	defPreferedRecSyntax;
	VISIBLE_STRING		defElementSetName;
	VISIBLE_STRING		displayFormats;
	VISIBLE_STRING		description;
	BIT_STRING		options;
	VISIBLE_STRING		contactInfo;
	BOOLEAN			defResultSetNamingSupported;
	SHORT			maxNumResultSets;
	SHORT			hoursOfOperation;
	VISIBLE_STRING		chargingPolicy;
	IR_LIST			dbInfo;		/* list of IR_DATABASE_INFO */
	VISIBLE_STRING		idAuthentication;
	VISIBLE_STRING		applCntxName;
	VISIBLE_STRING		applProcessTitle;
	VISIBLE_STRING		applEntityQual;
	VISIBLE_STRING		ipAddress;
	VISIBLE_STRING		ipDomainName;
	USHORT			portNum;
	BOOLEAN			isMalloced;
} OP_TARGET_INFO_STRUCT;

/*
 * Prototypes
 */
BOOLEAN 	OpCf_IsOpInitialized();
void 		OpCf_IROMark(void *function, void *data);
void		OpCf_IROFree();
void		OpCf_CommReplyFree(IRO_COMM_EVENT_REPLY *commReply);
IR_STATUS  	OpCf_GetString(AsnOcts *src, CHAR **dest);
IR_STATUS 	OpCf_UpdateErrorReport(IR_ERROR_LEVEL level,
				       IR_ERROR_CODE  code,
			       	       CHAR	      *source);
void		OpCf_ErrorInit(void);

IR_STATUS 	OpAm_AssocDelete(IR_ASSOC_ID assocId);
OP_ASSOC_ENTRY 	*OpAm_AssocGet(IR_ASSOC_ID assocId);
OP_ASSOC_ENTRY 	*OpAm_AssocCreate(IR_ASSOC_ID assocId);
void 		OpAm_AssocListFree(void);
IR_STATUS 	OpAm_AssocInit(void);
void		OpAm_AssocInitInfo(IRO_COMM_INIT_RESP *src,
		    IRO_COMM_INIT_RESP *dest);

IR_STATUS 	OpAm_AbortAssoc(IR_ASSOC_ID assocId);
IR_STATUS 	OpAm_ReleaseAssoc(IR_ASSOC_ID assocId);
IR_STATUS 	OpAm_ConnectAssoc(IR_ASSOC_ID assocId);

IR_STATUS 	OpAm_InitService(IR_ASSOC_ID assocId, APDU *apdu, 
	  		         IRO_COMM_INIT_RESP *initResp);
void 		OpAm_InitServiceFree(IRO_COMM_INIT_RESP *initResp);

IR_STATUS 	OpArc_AccessService(IR_ASSOC_ID assocId, APDU *apdu,
	    			IRO_COMM_ACCS_CNTL_REQ *accessReq);
void 		OpArc_AccessServiceFree(IRO_COMM_ACCS_CNTL_REQ *req);

IR_STATUS 	OpArc_RsrcCntlService(IR_ASSOC_ID assocId, APDU *apdu,
	  	  IRO_COMM_RSRC_CNTL_REQ *req);
void 		OpArc_RsrcControlServiceFree(IRO_COMM_RSRC_CNTL_REQ *req);
void 		OpAm_GetListFree(IRO_ASSOC_REPLY *reply);
BOOLEAN		OpAm_AssocServiceAllowed(SHORT, IR_ASSOC_ID);

IR_STATUS 	OpArc_RsrcReportService(IR_ASSOC_ID assocId, APDU *apdu,
	  	  IRO_COMM_RSRC_REPORT *reply);
void 		OpArc_RsrcReportServiceFree(IRO_COMM_RSRC_REPORT *resp);
void 		OpArc_ResourceReportFree(IRO_COMM_RSRC_REPORT *resp);
IR_STATUS 	OpArc_ResourceReport(ResourceReport *apdu, 
		    SHORT reportStatus, IRO_COMM_RSRC_REPORT *resp);
IR_STATUS	OpArc_VerifyRsrcRprtSyntaxOI(OBJECT_IDENTIFIER *oid);


IR_STATUS	OpTm_TargetAssocInfo(IR_TARGET_ID targetId, 
		    IR_TARGET_ASSOC_INFO *assocInfo);
void		OpTm_TargetAssocInfoFree(IR_TARGET_ASSOC_INFO *ai);
IR_STATUS 	OpTm_TargetInfo(IR_TARGET_ID targetId, 
		    IR_TARGET_INFO *assocInfo);
void 		OpTm_InitReqInfoFree(OP_INIT_REQ_INFO *initReqInfo);
IR_STATUS	OpTm_InitReqInfo(IR_TARGET_ID targetId, 
		    OP_INIT_REQ_INFO *reqInfo);

OP_SESSION_INFO *OpTm_UserSessionInfo(IR_USER_ID userId);

/* Search request functions */
IR_STATUS		OpSp_SearchService(IR_ASSOC_ID assocId, APDU *apdu,
					IRO_COMM_SRCH_RESP *searchResp);
void			OpSp_SearchServiceFree(IRO_COMM_SRCH_RESP *searchResp);
IR_STATUS		OpSp_ConvertRPNSearchQuery(IR_RPN_QUERY_STRUCT *reqQuery,
					RPNQuery **apduQuery);

void OpSp_DestroyDiagRec(IR_DIAG_RECORD *diagRec);
void OpSp_ConvertRecordList(IR_LIST *recList, RecordsSeqOf *apduList);
void OpSp_ConvertDiagRecToIR(IR_DIAG_RECORD *rec, DiagRec *apduDiag);
void OpSp_ConvertDbRecToIR(IR_DB_RECORD *rec, DatabaseRecord *apduRec);
void OpSp_RecordFree(CHAR *srcFile, void *data);
void *OpSp_RecordCopy(CHAR *srcFile, void *srcElem);

IR_STATUS	OpWs_Init(void);
OP_WS_INFO	*OpWs_WsCreate(IR_USER_ID);
OP_WS_INFO	*OpWs_WsGet(IR_WS_ID);
IR_STATUS	OpWs_WsDelete(IR_WS_ID);
void		OpWs_WsListFree(void);
void		OpWs_WsInit();
void		OpWs_WsDeleteAssoc(IR_ASSOC_ID);
void		OpWs_WsNewResultName(IR_ASSOC_ID, VISIBLE_STRING *, BOOLEAN);
SHORT		OpWs_WsFind(IR_ASSOC_ID);
IR_WS_ID	*OpWs_WsGetAllAssoc(IR_ASSOC_ID assocId);
void		OpWs_WsDeletedRemoteResult(IR_WS_ID);
void		OpWs_DeleteServiceFree(IRO_COMM_DEL_RESP *);
IR_STATUS	OpWs_DeleteService(IR_ASSOC_ID, APDU *, 
		    IRO_COMM_DEL_RESP *);
void		OpPm_PresentServiceFree(IRO_COMM_PRSNT_RESP *);
IR_STATUS	OpPm_PresentService(IR_ASSOC_ID, APDU *, 
		    IRO_COMM_PRSNT_RESP *);

IR_STATUS  	OpTm_GetUserTargets(VISIBLE_STRING fileName, 
					IR_LIST **targetInfo);
void		OpTm_UserLoginInfoFree(IRO_USER_LOGIN *reply);
void		OpTm_UserGenInfoFree(IR_LIST *reply);
void		OpTm_UserTargetInfoFree(IR_LIST *reply);
void		OpTm_TargetInfoFree(OP_TARGET_INFO_STRUCT *targetInfo);
IR_STATUS	OpTm_GetTargetFileName(IR_USER_ID userId,
					VISIBLE_STRING *targetInfoFileName);
IR_STATUS	OpTm_GetUserInfo(VISIBLE_STRING *key,
					OP_SESSION_INFO **info);
IR_STATUS	OpTm_GetNextUserInfo(OP_SESSION_INFO **info);
IR_STATUS	OpTm_GetTargetInfo(IR_TARGET_ID *targetId, 	
					VISIBLE_STRING targetInfoFileName, 
					RESC_IO_FILE_STRUCT *targetFilePtr,
					OP_TARGET_INFO_STRUCT **info);
IR_STATUS	OpTm_GetNextTargetInfo(RESC_IO_FILE_STRUCT *targetFilePtr,
					OP_TARGET_INFO_STRUCT **info);
VISIBLE_STRING	OpTm_WorkPath();
VISIBLE_STRING	OpTm_UserInfoFileName();
IR_STATUS	OpTm_Initialize();
IR_STATUS	OpTm_CloseUserInfoFile();
IR_STATUS	OpTm_CloseTargetInfoFile(RESC_IO_FILE_STRUCT *filePtr);
IR_STATUS	OpTm_ConvertUserInfoToRecord(OP_SESSION_INFO *info,
					VISIBLE_STRING **record);
IR_STATUS	OpTm_ConvertTargetInfoToRecord(OP_TARGET_INFO_STRUCT *info,
					VISIBLE_STRING **record);
IR_STATUS	OpTm_ConvertDbInfoToRecord(IR_DATABASE_INFO *info,
					VISIBLE_STRING **record);

IR_STATUS	OpTm_ConvertRecordToTargetInfo(VISIBLE_STRING *record,
					OP_TARGET_INFO_STRUCT **info);
IR_STATUS	OpTm_ConvertRecordToUserInfo(VISIBLE_STRING *record,
					OP_SESSION_INFO **info);
void		OpTm_UserTargetInfoElemFree(CHAR *srcFile, 
					IR_USER_TARGET_INFO *info);
void		OpTm_UserGenInfoElemFree(CHAR *srcFile, IR_USER_GEN_INFO *info);
void		OpTm_TargetGenInfoFree(IR_TARGET_INFO *reply);
IR_STATUS	OpTm_ConvertRecordToDbInfo(VISIBLE_STRING *record, 
					IR_DATABASE_INFO **info);	
IR_STATUS	OpTm_GetDBInfo(RESC_IO_FILE_STRUCT *filePtr, 
					IR_TARGET_ID targetId,
					IR_LIST *dbInfo);
void		OpTm_DBInfoFree(IR_LIST *reply);
void		OpTm_DBFree(CHAR *srcFile, IR_DATABASE_INFO *info);
void		OpTm_InitInfoFree(OP_INIT_REQ_INFO *info);
IR_USER_ID	OpTm_UserId();
CHAR		*OpTm_GetElem(VISIBLE_STRING record, ULONG start, ULONG length);
void		OpTm_PutElem(CHAR *start, USHORT tag, CHAR *elem, ULONG len,
					ULONG *end);
IR_DATABASE_INFO *OpTm_DBClone(CHAR *srcFile, IR_DATABASE_INFO *info);
IR_USER_GEN_INFO *OpTm_UserGenInfoClone(CHAR *srcFile, IR_USER_GEN_INFO *info);
IR_USER_TARGET_INFO *OpTm_UserTargetInfoClone(CHAR *srcFile, 
					IR_USER_TARGET_INFO *info);
void		OpTm_UserInfoInit(OP_SESSION_INFO *info);
void		OpTm_DbInfoInit(IR_DATABASE_INFO *info);
void		OpTm_TargetInfoInit(OP_TARGET_INFO_STRUCT *info);
void		OpTm_HexStrToByteStr(CHAR *buf);

#endif
