
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * HEADER resc.h
 *
 * PURPOSE:
 *	This header file contains the macros, data structures and external
 *	prototypes for RESC functions.
 *
 * SCOPE:
 *	Used by any unit that uses a resource function.
 *
 * NOTES:
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 * 
 ****************************************************************************/

#ifndef IR_RESC_H
#define IR_RESC_H
#include <ir.h>
#include <debug.h>
#include <string.h>

/****************************************************************************
* Macros: 
****************************************************************************/

CHAR 	*RescIR_ExpandFile(int, char *);

#define IR_FILE (RescIR_ExpandFile(__LINE__, __FILE__))

#if OLDSTUFF
#define IR_FILE (RescIR_ExpandFile(__LINE__, FILENAME))
#endif

#define VSToStrNullTerm(a,b,StrLen)\
    {\
	if (((b)->sPtr != (CHAR *)NULL) && ((b)->len != 0))\
	{\
		strncpy(a, (b)->sPtr, \
		    (StrLen < (b)->len ? StrLen : (b)->len)); \
		(a)[(StrLen < (b)->len ? StrLen : (b)->len)] = (CHAR)NULL;\
	}\
	else\
	{\
		(a)[0] = (CHAR)NULL;\
	}\
    }

#define LENPLUSNULL(sPtr) (strlen(sPtr)+1)

#define RESC_ASN_NO_MALLOC(asnPtr) (asnPtr)->isMalloced = FALSE
#define RESC_ASN_MALLOCED(asnPtr) (asnPtr)->isMalloced = TRUE
#define RESC_ASN_IS_MALLOCED(asnPtr) (asnPtr)->isMalloced

#define RESC_ASN_INIT_VS(asnPtr)\
		{\
			 (asnPtr)->len = 0;\
			 (asnPtr)->sPtr = (CHAR *)NULL;\
		}
#define RESC_ASN_INIT_OS(asnPtr)\
		{\
			 (asnPtr)->len = 0;\
			 (asnPtr)->sPtr = (CHAR *)NULL;\
		}
#define RESC_ASN_INIT_BS(asnPtr)\
		{\
			 (asnPtr)->len = 0;\
			 (asnPtr)->sPtr = (CHAR *)NULL;\
		}
#define RESC_ASN_INIT_OI(asnPtr)\
		{\
			 (asnPtr)->len = 0;\
			 (asnPtr)->sPtr = (CHAR *)NULL;\
		}

/****************************************************************************
 * Prototypes
 ****************************************************************************/

CHAR *RescIR_Malloc(CHAR *src, ULONG size);
CHAR *RescIR_Zmalloc(CHAR *src, ULONG size);
CHAR *RescIR_Realloc(CHAR *src, CHAR *curBlock, ULONG size);
CHAR *RescIR_Free(CHAR *src, CHAR *curBlock);
CHAR *RescIR_StrDup(CHAR *src, CHAR *curBlock);

VISIBLE_STRING		*RescASN_CreateVS(CHAR *srcFile);
OCTET_STRING		*RescASN_CreateOS(CHAR *srcFile);
BIT_STRING		*RescASN_CreateBS(CHAR *srcFile);
OBJECT_IDENTIFIER	*RescASN_CreateOI(CHAR *srcFile);
SHORT			*RescASN_CreateSI(CHAR *srcFile);
LONG			*RescASN_CreateLI(CHAR *srcFile);
void 			RescASN_DestroyVS(CHAR *srcFile, void *pData);
void 			RescASN_DestroyOS(CHAR *srcFile, void *pData);
void 			RescASN_DestroyBS(CHAR *srcFile, void *pData);
void 			RescASN_DestroyOI(CHAR *srcFile, void *pData);
void 			RescASN_DestroySI(CHAR *srcFile, void *pData);
void			RescASN_DestroyLI(CHAR *srcFile, void *pData);
IR_STATUS	RescASN_AddVS(CHAR *srcFile, VISIBLE_STRING 
		    *vs, CHAR *str, USHORT len);
IR_STATUS	RescASN_AddOS(CHAR *srcFile, OCTET_STRING *os, 
		    CHAR *str, USHORT len);
IR_STATUS	RescASN_AddBS(CHAR *srcFile, BIT_STRING *bs, 
		    CHAR *str, USHORT len);
IR_STATUS	RescASN_AddOI(CHAR *srcFile, OBJECT_IDENTIFIER *oi, 
		    CHAR *str, USHORT len);

IR_STATUS	RescASN_CopyVS(CHAR *srcFile, VISIBLE_STRING *dest, 
		    VISIBLE_STRING *src);
IR_STATUS	RescASN_CopyOS(CHAR *srcFile, OCTET_STRING *dest, 
		    OCTET_STRING *src);
IR_STATUS	RescASN_CopyBS(CHAR *srcFile, BIT_STRING *dest, 
		    BIT_STRING *src);
IR_STATUS	RescASN_CopyOI(CHAR *srcFile, OBJECT_IDENTIFIER *dest, 
		    OBJECT_IDENTIFIER *src);

void 		*RescASN_CloneVS(CHAR *srcFile, void *srcVs);
SHORT		RescASN_CompareVS(CHAR *srcFile, VISIBLE_STRING *v1,
		    VISIBLE_STRING *v2);
SHORT		RescASN_CmpOIToStrOI(CHAR *srcFile, OBJECT_IDENTIFIER *oi,
		    CHAR *str);

void		*RescUT_CloneBib1Attr(CHAR *srcFile, IR_ATTR_ELEM *srcAttr);

void		RescUT_DebugVS(CHAR *srcFile, VISIBLE_STRING *vs);
void		RescUT_ShortToVS(CHAR *src, VISIBLE_STRING *, SHORT);
SHORT		RescUT_VSToShort(CHAR *src, VISIBLE_STRING *);


SHORT		RescUT_DestroyIROSearchReq(void *);

void		RescUT_StartTime(CHAR *srcFile);
void		RescUT_EndTime(CHAR *srcFile);
#endif

