
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * HEADER ua_rdc.h
 *
 * PURPOSE:
 *	This header file is the general UA include file.  
 *
 * SCOPE:
 *	All UA CSCs must include this header file to ensure that the
 *	correct system wide types are being used.
 *	
 * NOTES:
 *	Character pointers specified in any structure element are null 
 *	terminated ASCII strings unless a count is associated with it.
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 * 
 ****************************************************************************/

#ifndef UA_RDC_H

#define UA_RDC_H
#include <ir.h>


/****************************************************************************
 * UA_RDC Common Constants and Types:
 ****************************************************************************/
/*
 * String size indicators.
 */
#define UARDC_USMARC_TAG_SIZE 	3	/* size of a USMARC tag */
#define UARDC_LINEBLK		20	/* Block of display lines */

/* Invalid USMARC tag. */
#define UARDC_USMARC_INVALID_TAG	""

/*
 * UARDC_FORMAT_STYLE: User Application Record Display Control Format
 * style indicates what sort of information will be displayed with
 * each field in a record.
 */
typedef CHAR	UARDC_FORMAT_STYLE;

/* UARDC_STYLE values. */
#define UARDC_STYLE_TAG		'T'	/* Each field displayed with tag */
#define UARDC_STYLE_LABLE	'L'	/* Each field displayed with lable */
#define UARDC_STYLE_NONE	'N'	/* Each field displayed unmarked */


/*
 * UARDC_FORMAT_TYPE: User Application Record Display Control Format
 * Type indiicates the format of the record.
 */
typedef CHAR	UARDC_FORMAT_TYPE;

/* UARDC_TYPE values. */
#define UARDC_TYPE_ALL		'A'	/* All fields displayed */
#define UARDC_TYPE_BRIEF	'B'	/* Only fields 245, 250, 260, 300 */ 
#define UARDC_TYPE_UMULTI	'M'	/* User defined multi lined */
#define UARDC_TYPE_UBRIEF	'U'	/* User defined single line. */


/*
 * UARDC_LINE_SIZE: User Application Record Display Control Line Size
 * indicates the maximum number of characters displayable on a single
 * line and is used to determine how to format a multi lined record.
 */
typedef SHORT	UARDC_LINE_SIZE;

/*
 * UARDC_TRUNKATE_AT: User Application Record Display Control  Trunkate
 * at type indicates at how many characters are allowed before
 * truncation should take place.  This is only used when the record
 * type is UBRIEF or BRIEF.  If you want no trunkation set the value to
 * zero.
 */
typedef SHORT	UARDC_TRUNKATE_AT;

/*
 * UARDC_USMARC_TAG: User Application Record Display Control USMARC 
 * Tag is a string to maintain a USMARK tag.
 */
typedef CHAR	UARDC_USMARC_TAG[UARDC_USMARC_TAG_SIZE +1];

/*
 * UARDC_USMARC_FORMAT_LIST: User Application Record Display Control USMARC
 * Format List is used to maintain a list of tags and lables  in the order
 * in which the format is to take place.
 */ 
typedef struct {
	UARDC_USMARC_TAG tag;
	CHAR 		 *lable;
} UARDC_USMARC_FORMAT_LIST;

/*
 * UARDC_USMARC_RECORD: User Application Record Display Control USMARC
 * Record contains a USMARC formatted record.
 */
typedef CHAR *UARDC_USMARC_RECORD;

/*
 * UARDC_FORMAT_LIST: User Application Record Display Control Format
 * List is used to maintain a list of fields to obtain from a record
 * based on the format.
 */
typedef union {
	UARDC_USMARC_FORMAT_LIST *usmarcList;
}UARDC_FORMAT_LIST;


/*
 * UARDC_RECORD_FORMAT_STRUCT: The User Application Record Display
 * Control Record Format Structure describes the format that should
 * determine the apperance of a record. 
 */
typedef struct {
	SHORT			rType;  /* General type of the record. */
	UARDC_FORMAT_STYLE	style;	/* TAGGED, LABLLED, NONE */
	UARDC_FORMAT_TYPE	type;	/* FULL/BRIEF/USER? */
	UARDC_LINE_SIZE		lineSz;	/* Size of a displayable line.  */
	UARDC_TRUNKATE_AT	trunkAt;/* Point to trunkate a line.  */
	UARDC_FORMAT_LIST	l;	/* List of Tags in order of display */
} UARDC_RECORD_FORMAT_STRUCT;


/*
 * UARDC_RECORD_STRUCT: The User Application Record Display
 * Control Record Format Structure describes the format that should
 * determine the apperance of a record. 
 */
typedef union {
	UARDC_USMARC_RECORD usmarc;
}UARDC_RECORD_STRUCT;

/*
 * UARDC_DISPLAY_STRUCT: The User Application Record Display
 * Control Record Format Structure describes the format that should
 * determine the apperance of a record. 
 */
typedef struct{
	USHORT	lines;
	CHAR 	**displayLines;
} UARDC_DISPLAY_STRUCT;

#endif /* UA_RDC_H */
